/*
 *  ia64/elf.c
 *
 *  Copyright (C) 2004 FUJITSU LIMITED
 *
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/* $Id: elf.c,v 1.3 2004/06/06 22:49:22 tachino Exp $ */

#include <glib.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/statfs.h>
#include <errno.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#include "../diskdumpelf.h"
#include "../dumpdevice.h"

void *get_task_struct_note(DumpDevice *device, int *note_len)
{
	struct disk_dump_header *header = device->header;
	struct switch_stack **sw = device->sub_header->sw;
	struct task_struct **tasks = device->header->tasks;
	guint64 *task_data;
	int i, j;
	int len = sizeof(guint64 *) * (header->nr_cpus * 2);

	task_data = g_malloc0(len);

	/* NT_TASKSTRUCT note
	 *
	 * format:
	 *    struct task_struct  *task;
	 *    struct switch_stack *sw;
	 *    struct task_struct  *other_task[nr_cpus-1];
	 *    struct switch_stack *other_sw[nr_cpus-1];
	 */
	task_data[0] = (guint64)tasks[header->current_cpu];
	task_data[1] = (guint64)sw[header->current_cpu];
	for (i = 0, j = 2; i < header->nr_cpus; i++) {
		if (i == header->current_cpu)
			continue;
		task_data[j] = (guint64)tasks[i];
		task_data[j+header->nr_cpus-1] = (guint64)sw[i];
		j++;
	}

	*note_len = len;
	return task_data;
}
